import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil
from common import *
#from nco import Nco


class DataS4(DataMod):
     """ classe of data for ECMWF S4
     """
     def __init__(self, dataname, varname):
         DataMod.__init__(self, dataname, varname)
         self._nmember=51
         self._freq=datetime.timedelta(0, 0, 0, 0, 0, 6)
         self._filetype="nc"

     def set_inputdirAuto(self):
         """defining the path to look for the model daily data"""
         dataname=self.get_dataname()
         varname=self.get_varname()
         
         suff="_6hourly"
         inputdir = "/cfunas/exp/ECMWF/system4/6hourly/" + varname +"/nc/"

         if (not os.path.isdir(inputdir)):
             print "This directory does not exist:"
             print inputdir
             sys.exit(0)

         self.set_inputdir(inputdir)

     def set_outputdirAuto(self):
          """defining the path to look for the model daily data"""
          dataname=self.get_dataname()
          varname=self.get_varname()
         
          suff="_6hourly"
          outputdir = "/cfunas/exp/ECMWF/system4/monthly_statistics/"

          if (not os.path.isdir(outputdir)):
               os.mkdir(outputdir)
               print "Created directory:"
               print outputdir

          self.set_outputdir(outputdir)

     def set_files(self, date1, date2):
        """create the Filelist and sdates argument with all files
        between date1 and date2 separated by one year
        check if these files exist
        check their type"""
        
        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]

        filebase=self.get_varname() + "_" + str(date1).replace("-","")+".nc"
        print(filebase)
        #look for the first file
        filetype=self.get_filetype()
        filename=glob.glob(inputdir+filebase)
        if len(filename)== 0:
            #case for file with the day not in the file name
            filebase=filebase.replace("01.nc",".nc")
            filename=glob.glob(inputdir+filebase)
            if len(filename)== 0:
                 print("no nc file in "+inputdir)
                 return 1

        #store the file
        filelist.append(filename[0])
        datelist.append(date1)
        date1 = date1 + relativedelta(years=+1)
        d=date1
        #loop on the date to find all the files
        while d <= date2:
            filebase=self.get_varname() + "_" + str(d).replace("-","")+".nc"
            filename=glob.glob(inputdir+"/"+filebase)
            if len(filename)== 0:
                 #case for file with the day not in the file name
                 filebase=filebase.replace("01.nc",".nc")
                 filename=glob.glob(inputdir+filebase)
                 if len(filename)== 0:
                      print "the file %s does not exit " % inputdir+filebase+"."+filetype
                      print "while the file %s exists "  % inputdir+filelist[0]
                      print "all file must have the same type (grb or nc)"
                      return 1

            datelist.append(d)
            filelist.append(filename[0])
            d = d + relativedelta(years=+1)

        self.set_filelist(filetype)
        self.set_filelist(filelist)
        self.set_sdates(datelist)
        return 0


     def format_files_nc(self):
         """transform the grb file 
         into ncdf file readable for cdo"""
         
         cdo=Cdo()
         #nco = Nco()

         #get the needed variables from the object
         filelist=self.get_filelist()
         sdates=self.get_sdates()
         dataname=self.get_dataname()
         varname=self.get_varname()
         nmember=self.get_nmember()
         filetype=self.get_filetype()

         #in ECMWF S4 the leadtime is 7months
         #hard coded
         nbmon=relativedelta(months=+7, days=-1)
         #
         #if varname in ["lsp", "cp"]:
         #     self.set_freq(datetime.timedelta(0, 0, 0, 0, 0, 24))

         freq=self.get_freq()

         for ifile in range(len(filelist)):
             f=filelist[ifile]
             d1=sdates[ifile]
             
             #check if the output file is already here
             out="format_"+varname+"_"+str(d1).replace("-","")+".nc"
             if len(glob.glob(out))==0:
                  
                  flocal=f.split("/")[-1]
                  #copy the file in the scratch (faster)
                  if len(glob.glob(flocal))==0:
                       shutil.copy(f,".")
                  f=flocal

                  #calculate the last date of the file
                  d2=d1+nbmon
            
                  #calculate the number of timestep
                  if freq.seconds != 0:
                       nbstep=((d2-d1).days + 1)*86400/freq.seconds
                  else:
                       nbstep=((d2-d1).days + 1) + 1
             


                  #list to store the members
                  lstmemb=[]
                  #loop over the member
                  for imemb in range(nmember):
                       print(imemb)
                       #step=[i * nmember +imemb for i in range(nbstep)]
                       step=list(numpy.array(range(nbstep))+nbstep*imemb)
                       #print(step)
                       #convert step in string
                       step=str(step).strip('[]').replace(" ","")
                       #select one member
                       fmemb=f+str(imemb)
                       print(fmemb)
                       cdo.seltimestep(step,  
                                       input=f, output=fmemb, options="-r")
                       lstmemb.append(fmemb)

                       #print(lstmemb)
                  #cat the ensemble members in a a new dimension
                  lstmembstr=str(lstmemb).strip('[]').replace(", "," ").replace("'","")

                  dictvar={"tas":"T2M","lsp":"LSP","cp":"CP","psl":"MSL",
                           "uas":"U10M","vas":"V10M"}
                  oldname=dictvar.get(varname)
                  if oldname == None:
                       print("the variable "+varname+"is no available in the format_file function")
                       sys.exit(0)
                  cdo.chvar(oldname,varname, 
                            input=cdo.merge(input=lstmembstr,output=out+"tmp",
                                            options="-r", 
                                            force =  False),
                            output=out, options="-r", 
                            force =  False)
                  #remove the file per member to save space
                  os.remove(out+"tmp")
                  for filetmp in lstmemb:
                       os.remove(filetmp)
                  #remove the copy of the original file to save space
                  os.remove(flocal)

                  print(out)
             filelist[ifile]=out
         self.set_filelist=list(filelist)
       
     def ready(self, date1, date2):
         """ make the Data object ready to calculate statistic
         """
         cdo=Cdo()

         dataname=self.get_dataname()
         varname=self.get_varname()

         if varname == "tasmin":
              self.set_varname("tas")
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.cdo_statistic("daymin","tasmin")
              self.see()
         elif varname == "tasmax":
              self.set_varname("tas")
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.cdo_statistic("daymax","tasmax")
         elif varname == "prlr":
              auxlsp=DataEC23(dataname,"lsp")
              auxlsp.set_inputdirAuto()
              auxlsp.set_files(date1,date2) #look for the files between date1 and date2 
              auxlsp.format_files_nc() #transform grib in nc files cdo compatible

              auxcp=DataEC23(dataname,"cp")
              auxcp.set_inputdirAuto()
              auxcp.set_files(date1,date2) #look for the files between date1 and date2 
              auxcp.format_files_nc() #transform grib in nc files cdo compatible

              aux=weighted_sum((auxlsp, auxcp), (1,1), varname)
              aux.set_outputdirAuto() #look for the output directory
              self.copy(aux)
         elif varname == "sfcwind":
              #test if the file are already in the workdir
              self.set_inputdir("./")
              test=self.set_files(date1,date2)
              self.see()
              if test == 1:
                                 
                   auxuas=DataS4(dataname,"uas")
                   auxuas.set_inputdirAuto()
                   auxuas.set_files(date1,date2) #look for the files between date1 and date2 
                   auxuas.see()
                   auxuas.format_files_nc()
                   auxuas.see()

                   auxvas=DataS4(dataname,"vas")
                   auxvas.set_inputdirAuto()
                   auxvas.set_files(date1,date2) #look for the files between date1 and date2 
                   auxvas.see()
                   auxvas.format_files_nc()
                   auxvas.see()
         
                   self.copy(module(auxuas,auxvas, "sfcwind"))
              self.see()
              self.set_outputdirAuto() #look for the output directory
              
         else:
              self.set_varname(varname)
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.see()
